'NoteKeeper 3                                           
'uses Treeview control to display topics and notes  
' - depends on the Treeview control itself to track and display topics and notes
' - Tview(), noteTitles() used only for saving treeview to file                                                                 
' - noteTexts() maintained during session (keeping deletions)
' - noteTexts() deletions are compressed during save
' - numItems, dim of noteTexts()... includes deletions
' - numTree, number of nodes in tree (numTree <= numItems)                                                  
'
'ver 3.00 (03/11/2010)
' - notes revised as tree structure of topics and notes (text shown in right hand pane)
'ver 3.04 (04/12/2010)
' - fixed moving notes feature
' - can now move topics as well as notes
'ver 3.05 (05/09/2010)
' - corrected error in MoveTo code
'ver 3.06 (05/25/2010)
' - changed data storage from _DOCUMENTS to _APPDATA
'ver 3.10 (06/14/2010)
' - changed file encryption to note encryption
' - corrected title of printed note
' - corrected some bugs associated with note protection
' - corrected some bugs associated with delete
' - streamlined code
'ver 3.20 (08/10/2010)
' - improved protected data with more consistant responses
' - removed ability to print a protected item without first verifying
' - added ability to change or remove password with new 'Password' File menu item
' - grayed 'moveTo' menu item rather than checked 'moveItem'
'ver 3.21 (09/09/2010)
' - a few improvements and corrections to the Find feature
'ver 3.25 (09/18/2010)
' - changed to a simpler method of showing hand cursor in 'About'                                                               
' - changed to simpler 'Replace' code, not requiring myInput box
' - changed to simpler 'AddTopic' and 'AddNote', not requiring another dialog box
'ver 3.26 (09/19/2010)
' - corrected problem with beginning new file
' - corrected problem updating statusbar after label edit
'ver 3.27 (10/15/2010)
' - corrected "not responding" problem with saving file                                                                          
' - corrected problem using ESC and ENTER while adding note or topic
'ver 3.30 (03/31/2011)                                                                                                           
' - corrected problem with protecting an item by marking it protected but not encrypting it
' - corrected various problems associated with searching
'ver 3.31 (04/10/2011)
' - added keyboard Shft+F10 to display popup menu on selected item                         
' - improved popup menu action on protected items
' - show protected items with "*" in status bar
' - added popup menu to right window textbox
'ver 3.32 (04/26/2011)                                                                                                           
' - corrected removal of note when clicking after right click on treeview                   
'ver 3.33 (06/24/2011)
' - corrected fault allowing focus on textbox when protected item not verified
' - corrected fault clicking [+] part of protected topic not verified
'ver 3.34 (06/27/2011)
' - added eight tabs to textbox and a keyboardHook function to process them
' - corrected CTRL+A to select all of textbox contents
'ver 3.35 (02/16/2012)
' - corrected problem with right click in blank area of tree (test hItem = 0)
' - should not select all when tabbing into texbox
' - should not be able to enter textbox of protected tree item before password verified
' - removed MANIFEST resource (I don't think User Account Control is necessary here)    
'ver 3.36 (03/09/2012)
' - fixed PB10 problem of bad topic/note editing (%TVN_BEGINLABELEDITW, %TVN_ENDLABELEDITW)
' - changed second click on a topic/note to not edit (must use Rename menu item)
'ver 3.37 (01/15/2016)   
' - corrected search for text found in locked item... couldn't open items, or start new search
' - added 'Enter password' to 'password' menu under 'File'
' - expanded search to facilitate beginning at top and entering password if needed
'ver 3.40 (01/16/2016)
' - removed checkbox for password from srchFor() since it can just be cancelled later
' - show secured content as soon as password is provided
' - new file no longer requires YES/NO/CANCEL prompt when exiting
' - improved search routines (still find next/previous after none found)

'NoteKeeper 4
'ver 4.00 (02/03/2017) 
' - changed to a simpler file format, keeping text array in sequential order
' - problem of deleting an item and having the next item lose its text data
' - problem of clicking on protected item and cancelling (text data is hash)
' - removed Protected() array, using Tview().protected instead
'ver 4.01 (03/08/2017)
' - put noteKeeper icon in explorer
' - corrected problem inserting first note of a topic (use %TVI_FIRST, not %TVI_LAST)
' - corrected problem deleting multiple notes (quit responding)
' - corrected problem with using emergency password for File > password
'ver 4.02 (04/07/2018)
' - corrected problem not holding unProtected after saving and re-entering program
' - corrected problem with entries remaining bold after selecting another entry
'ver 4.03 (06/01/2018)                                                                   
' - set dataFolder to folder that NoteKeeper.exe is launched from and contains *.nkp data file
' - corrected problems with deleting and moving items (untrapped error #9) 
' - corrected problems with saving data file (saveTreeview)
'ver 4.04 (10/17/2018)
' - corrected problem with adding a note or a topic... program closes
'ver 4.05 (11/14/2018)
' - now use F2 to toggle between treeview and textbox windows instead of tab
' - now can use tab key for actually tabbing when inside the textbox (used Ctrl+Tab before)
' - remove keyboardHook... never needed it anyway
'ver 4.06 (06/07/2019)
' - inserted single line textbox at top to serve as title for tree item                                            
' - corrected object of pointer, sFind (local to global)
' - hilite tree items as search finds the items
' - corrected moving topics and notes and added ability to change level 
' - added wholeword checkbox to search 
' - improved getNextItem() and getPrevItem() routines and corrected problems with search
'ver 4.07 (06/19/2019)
' - fixed adding and deleting entries               
' - fixed the end of editing entry name
'ver 4.08 (08/03/2019)
' - fixed adding topic after a topic with notes (needed to add after all the notes)
'ver 4.09 (10/10/2019)
' - fixed problem with saving after a delete... needed to include showTreeView(0)
'ver 5.00 (11/13/2019) 
' - attempt to make noteKeeper and genHelp as close as possible with regard to the treeview
' - before this attempt, cleaned up as many bugs in the treeview side of genHelp as possible
' - the main difference between noteKeeper and genHelp is the protection feature in noteKeeper
' - added helpfile to noteKeeper                                            
'------------------------------------------------------------------------------------------------------
              
%IDD_DIALOG1      =  100
%IDC_Treeview     =  101
%IDC_Title        =  102                            
%IDC_Textbox      =  103
%IDC_Splitbar     =  104
%IDC_VertLine     =  105                                
%IDC_Statusbar    =  106

%IDC_EnterPW      =  121
%IDC_Prompt       =  122
%IDC_RemovePW     =  123
%IDC_ChangePW     =  124
%IDC_Password     =  125
%IDC_Confirm      =  126
%IDC_PasswordHlp  =  127

%IDC_TopicOpt     =  201
%IDC_NoteOpt      =  202
%IDC_AddName      =  203
%IDM_New          =  301
%IDM_Open         =  302
%IDM_Save         =  303
%IDM_SaveAs       =  304
%IDM_Protect      =  305
%IDM_Password     =  306
%IDM_Exit         =  307

%IDM_AddTopic     =  401
%IDM_AddNote      =  402
%IDM_Rename       =  403
%IDM_ChangeLevel  =  404
%IDM_Move         =  405
%IDM_MoveTo       =  406
%IDM_DeleteItem   =  407
%IDM_Print        =  408
%IDM_ExpandAll    =  409                           
%IDM_Collapse     =  410

%IDM_Undo         =  501
%IDM_Cut          =  502
%IDM_Copy         =  503
%IDM_Paste        =  504
%IDM_Delete       =  505
%IDM_SelAll       =  506
%IDM_Tab          =  507
%IDM_TabSpaces    =  521
%IDM_WordWrap     =  522

%IDM_Find         =  601        
%IDM_Next         =  602                      
%IDM_Prev         =  603
%IDM_OtherPane    =  604
            
%IDM_GenHelp      =  701                        
%IDM_About        =  702

type editOption
   tabSpaces as long                                                        
   nWordWrap as long                                                    
end type

TYPE TreeviewType
   level AS LONG       '0 if topic, 1 if note
   protected AS LONG   '0=unprotected, 1=protected(encrypted), -1=protected(not encrypted)
END TYPE

GLOBAL hDlg, hFont, numItems, numTree, fileChanged, nWordWrap, lastWordwrap AS LONG
GLOBAL hMenu, hFileMenu, hItemMenu, hEditMenu, hSrchMenu, hHelpMenu as long
GLOBAL hTextbox, hTitle, editLabel, origTextboxProc AS LONG
global hTreeviewMenu, hTextboxMenu, tabSpaces, inputOpt AS LONG 
global wholeWord, doNotNotify, numProtected, verified, redoTreeHandles as long
GLOBAL dataFolder, iniFile, thisFile, pwFile AS STRING
GLOBAL titles(), texts() AS STRING 
global treeHandles() as long
global editOpt() as editOption
global TView() as TreeviewType 
'global dbug as long

'---------------------------------------------------------------------------------------
'Notes:
'   numItems = number of items in texts(), titles(), treeLevel(), editOpt()... never less than first opened
'   numTree  = current number of tree items in treeHandles()... will be less than numItems with deletions                                         
'
'   texts()  = help contents (stored sequentially)
'   titles() = help titles (stored sequentially)
'   treeLevel()  = 0 if item i is a topic
'                = 1 if item i is a note                                   
'                  (stored sequentially)
'   savedIndexes() = indexes pointing to helpContents(), not reqd with revised strategy
'                    still may be used for compatibiliy with original strategy
'                    (stored in tree view order)
'
'   dataFolder = CSIDL_COMMON_APPDATA when program folder is CSIDL_COMMON_PROGRAMS
'              = program folder, otherwise
'
'   wordWrap =  1 on
'            =  0 off
'

